/*
 *	led.c
 */

#include "dinv.h"

#define WHITE	RGB(31,31,31)
#define BLACK	RGB(0,0,0)

typedef struct _BMP {
	int	xsize, ysize;
	u16	*im;
} BMP, *PBMP;

static const u16	imTop[] = {
	0x7f80,
	0xffc0,
	0x7f80,
	0x3f00,
};

static const BMP	bmTop = { 10, 4, (u16 *)imTop };

static const u16	imMiddle[] = {
	0x3f00,
	0x7f80,
	0xffc0,
	0x7f80,
	0x3f00,
};

static const BMP	bmMiddle = { 10, 5, (u16 *)imMiddle };

static const u16	imBottom[] = {
	0x3f00,
	0x7f80,
	0xffc0,
	0x7f80,
};

static const BMP	bmBottom = { 10, 4, (u16 *)imBottom };

static const u16	imLeft[] = {
	0x4000,
	0xe000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xe000,
	0x4000,
};

static const BMP	bmLeft = { 4, 13, (u16 *)imLeft };

static const u16	imRight[] = {
	0x2000,
	0x7000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0xf000,
	0x7000,
	0x2000,
};

static const BMP	bmRight = { 4, 13, (u16 *)imRight };

#define SCREEN_OFFSET(x, y)	((x) + (y) * SCREEN_WIDTH)

typedef struct _LED {
	int	offset;
	PBMP	pbm;
} LED, *PLED;

static const LED	LEDs[7] = {
	{ SCREEN_OFFSET(3, 0), (PBMP)&bmTop },
	{ SCREEN_OFFSET(1, 2), (PBMP)&bmLeft },
	{ SCREEN_OFFSET(11, 2), (PBMP)&bmRight },
	{ SCREEN_OFFSET(3, 13), (PBMP)&bmMiddle },
	{ SCREEN_OFFSET(1, 16), (PBMP)&bmLeft },
	{ SCREEN_OFFSET(11, 16), (PBMP)&bmRight },
	{ SCREEN_OFFSET(3, 27), (PBMP)&bmBottom },
};

#define MAX_LEDCOLS	8
#define LED_OFFSET	24

static u8	LEDScrn[MAX_LEDCOLS];
static int	BaseX, BaseY;

/*
 *	LED pattern definition
 */

#define LED0	0x77
#define LED1	0x12
#define LED2	0x5d
#define LED3	0x5b
#define LED4	0x3a
#define LED5	0x6b
#define LED6	0x6f
#define LED7	0x72
#define LED8	0x7f
#define LED9	0x7b
#define LEDn	0x0e

static const u8	LEDPtn[] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 00
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x09, 0x49, 0x00, 0x00, 0x00, 0x00, 0x00,	// 10
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 20
	0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00,
	LED0, LED1, LED2, LED3, LED4, LED5, LED6, LED7,	// 30
	LED8, LED9, LEDn, 0x00, 0x00, 0x00, 0x00, 0x00,
};

#define MAX_LEDPTN	(sizeof(LEDPtn) / sizeof(LEDPtn[0]))

/*
 *	InitLED
 */
void InitLED(int x, int y)
{
	int		i;

	for (i = 0; i < MAX_LEDCOLS; i++)
		LEDScrn[i] = 0;

	BaseX = x;
	BaseY = y;
}

/*
 *	PutLED
 */
void PutLED(int offset, int ledno)
{
	u8		ledptn;
	int		i;
	int		x0, y0;
	int		x, y;
	u16		ptn;
	vu16		*v0, *v1, *v;
	PLED		pl;
	PBMP		pbm;
	u16		*pim;
	u16		bmptn;

	if (offset >= MAX_LEDCOLS || ledno >= MAX_LEDPTN)
		return;

	ledptn = LEDPtn[ledno];
	x0 = BaseX + offset * LED_OFFSET;
	y0 = BaseY;
	v0 = (vu16 *)VRAM_ADDR + (x0 + y0 * SCREEN_WIDTH);
	for (i = 0, ptn = 0x40, pl = (PLED)LEDs; i < 7; i++, ptn >>= 1, pl++) {
		if ((LEDScrn[offset] & ptn) == (ledptn & ptn))
			continue;
		v1 = v0 + pl->offset;
		pbm = pl->pbm;
		pim = pbm->im;
		for (y = 0; y < pbm->ysize; y++) {
			v = v1;
			bmptn = 0x8000;
			if (ledptn & ptn) {
				for (x = 0; x < pbm->xsize; x++, bmptn >>= 1) {
					if (*pim & bmptn)
						*v = WHITE;
					v++;
				}
			} else {
				for (x = 0; x < pbm->xsize; x++, bmptn >>= 1) {
					if (*pim & bmptn)
						*v = BLACK;
					v++;
				}
			}
			v1 += SCREEN_WIDTH;
			pim++;
		}
	}

	LEDScrn[offset] = ledptn;
}

/*
 *	PutLEDStr
 */
void PutLEDStr(int offset, u8 *s)
{
	while (*s)
		PutLED(offset++, *s++);
}

/*
 *	ClearLED
 */
void ClearLED()
{
	int		i;

	for (i = 0; i < MAX_LEDCOLS; i++)
		PutLED(i, 0);
}

