/*
 *	sprintf.c
 */

#include <stdarg.h>
#include "dinv.h"

static const u8	ItoA[] = "0123456789ABCDEF";

/*
 *	PutValue
 */
static u8 *PutValue(u8 *bp, s32 v, s32 base, u8 sign, u8 len, u8 sc)
{
	u32		uv;
	u8		*vp;
	u8		n;
	u8		vbuf[16];

	if (sign && v < 0) {
		*bp++ = '-';
		uv = (u32)-v;
	} else
		uv = v;

	vp = vbuf;
	n = 0;
	do {
		*vp++ = ItoA[uv % base];
		uv /= base;
		n++;
	} while (uv > 0);
	if (len)
		while (n < len && vp < &vbuf[16]) {
			*vp++ = sc;
			n++;
		}
	while (vp > vbuf)
		*bp++ = *--vp;

	return bp;
}

/*
 *	_sprintf
 */
void _sprintf(u8 *bp, u8 *fmt, ...)
{
	va_list		vp;
	u8		c;
	u8		sc;
	u8		len;
	s32		v;
	u8		*s;

	va_start(vp, fmt);
	while (c = *fmt++) {
		if (c != '%') {
			*bp++ = c;
			continue;
		}

		sc = ' ';
		len = 0;
	format:
		switch (c = *fmt++) {
		default:
			*bp++ = c;
			break;
		case '0':
			sc = c;
			goto format;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			len = len * 10 + (c - '0');
			goto format;
		case 'd':
			v = va_arg(vp, s32);
			bp = PutValue(bp, v, 10, TRUE, len, sc);
			break;
		case 'x':
			v = va_arg(vp, u32);
			bp = PutValue(bp, v, 16, FALSE, len, sc);
			break;
		case 's':
			s = va_arg(vp, u8 *);
			while (*s)
				*bp++ = *s++;
			break;
		}
	}
	*bp++ = '\0';
	va_end(vp);
}

