/*
 *	sram.c
 */
#include <gba.h>
#include "dinv.h"

typedef struct _SAVEDATA {
	vu8	Sig[2];
	vu8	Ver[2];
	vu8	HighScore[4];
} SAVEDATA, *PSAVEDATA;

//#define	pSaveData	((PSAVEDATA)SRAM_ADDR)
#define	pSaveData	((PSAVEDATA)((u8 *)SRAM_ADDR + 0x1000)) // for GBA filer

#define	DINV_SIG	"DI"
#define	DINV_VER	0x0001

/*
 *	IsSigOK
 */
static BOOL IsSigOK()
{
	return	pSaveData->Sig[0] == DINV_SIG[0] &&
		pSaveData->Sig[1] == DINV_SIG[1];
}

/*
 *	LoadHighScore
 */
u32 LoadHighScore()
{
	if (!IsSigOK())
		return 0;

	return	(pSaveData->HighScore[0]) |
		(pSaveData->HighScore[1] << 8) |
		(pSaveData->HighScore[2] << 16) |
		(pSaveData->HighScore[3] << 24);
}

/*
 *	SaveHighScore
 */
void SaveHighScore(u32 HighScore)
{
	if (!IsSigOK())
		return;

	pSaveData->HighScore[0] = (u8)HighScore;
	pSaveData->HighScore[1] = (u8)(HighScore >> 8);
	pSaveData->HighScore[2] = (u8)(HighScore >> 16);
	pSaveData->HighScore[3] = (u8)(HighScore >> 24);
}

/*
 *	InitSRAM
 */
void InitSRAM()
{
	if (IsSigOK())
		return;

	pSaveData->Sig[0] = DINV_SIG[0];
	pSaveData->Sig[1] = DINV_SIG[1];
	pSaveData->Ver[0] = LOBYTE(DINV_VER);
	pSaveData->Ver[1] = HIBYTE(DINV_VER);

	SaveHighScore(0);
}

