/*
 *	subr.c
 */

#include "dinv.h"

static u32	RandomSeed;

#define KEY_REPEAT_TIME	500
#define KEY_WAIT	10

static u16	LastKey;
static u32	WaitTimer;
static u32	RepeatTimer;

/*
 *	GetKey
 */
int GetKey()
{
	u16		key;

	if (ClockTick - WaitTimer < KEY_WAIT)
		return 0;

	key = (~READ_REG16(KEY_STS)) & KEY_ALL;
	WaitTimer = ClockTick;

	if (key && key == LastKey) {
		if (ClockTick - RepeatTimer < KEY_REPEAT_TIME)
			return 0;
		return KEY_REPEAT | key;
	}

	LastKey = key;
	RepeatTimer = ClockTick;

	if (key & KEY_SELECT) {
		// RESET
		void (*preset)() = (void (*)())ROM_ADDR;
		(*preset)();
	}

	return key;
}

/*
 *	InitKey
 */
void InitKey()
{
	LastKey = 0;
	WaitTimer = RepeatTimer = ClockTick;
}

/*
 *	Srand
 */
void Srand(u32 seed)
{
	RandomSeed = seed;
}

/*
 *	Rand
 */
u32 Rand()
{
	return RandomSeed = (RandomSeed * 23514) * 15178 / 3195;
}

/*
 *	FlashTextAndWait
 */
void FlashTextAndWait(int x, int y, u8 *str, u16 keymask, u32 timeout)
{
	static const u8	sBlank[] = "                    ";

	while (GetKey())
		;

	if (timeout != INFINITE)
		timeout += ClockTick;

	for (;;) {
		if (timeout != INFINITE && ClockTick > timeout)
			break;
		Put8Str(str, x, y);
		if (KeyWait(keymask, 500))
			break;
		Put8Str((u8 *)sBlank, x, y);
		if (KeyWait(keymask, 100))
			break;
	}
	Put8Str((u8 *)sBlank, x, y);
}

